@@ -24,13 +24,14 @@ from collections import namedtuple
 from collections import OrderedDict
 import copy
 import datetime
-import hashlib
 import inspect
 import logging
-import mock
-from oslo_utils import versionutils as vutils
+from unittest import mock
 
 import fixtures
+from oslo_utils.secretutils import md5
+from oslo_utils import versionutils as vutils
+
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
@@ -270,8 +271,9 @@ class ObjectVersionChecker(object):
         if extra_data_func:
             relevant_data += extra_data_func(obj_class)
 
-        fingerprint = '%s-%s' % (obj_class.VERSION, hashlib.md5(
-            bytes(repr(relevant_data).encode())).hexdigest())
+        fingerprint = '%s-%s' % (obj_class.VERSION, md5(
+            bytes(repr(relevant_data).encode()),
+            usedforsecurity=False).hexdigest())
         return fingerprint
 
     def get_hashes(self, extra_data_func=None):
